import isString from 'lodash/isString';
import partial from 'lodash/fp/partial';
import request, { makeParams } from '../utils/request';
import post from '../utils/post';
import config from '../utils/config';
import { getDomain } from '../utils/auth';

export const datasourceApi = (coordinate) => {
  const { containerType, containerName, datasourceName } = isString(coordinate) ? {
    containerType: 'global',
    datasourceName: coordinate,
  } : (coordinate || {});
  if (containerType === 'global') {
    return {
      query: partial(calcGlobalDatasource, [datasourceName]),
      queryMeta: partial(calcGlobalDatasourceMeta, [datasourceName]),
      count: partial(countGlobalDatasource, [datasourceName]),
      countMeta: partial(countGlobalDatasourceMeta, [datasourceName]),
      cursor: partial(cursorGlobalDatasource, [datasourceName]),
      cursorMeta: partial(cursorGlobalDatasourceMeta, [datasourceName]),
      update: partial(updateGlobalDatasource, [datasourceName]),
      updateMeta: partial(updateGlobalDatasourceMeta, [datasourceName]),
      create: partial(createGlobalDatasource, [datasourceName]),
      createMeta: partial(createGlobalDatasourceMeta, [datasourceName]),
      remove: partial(removeGlobalDatasource, [datasourceName]),
      removeMeta: partial(removeGlobalDatasourceMeta, [datasourceName]),
      meta: partial(getGlobalDatasourceMeta, [datasourceName]),
      validateUpdate: partial(validateUpdateGlobalDatasource, [datasourceName]),
      validateCreate: partial(validateCreateGlobalDatasource, [datasourceName]),
      validateRemove: partial(validateRemoveGlobalDatasource, [datasourceName]),
      feature: partial(getGlobalDatasourceFeature, [datasourceName]),
      invoke: partial(invokeGlobalDatasource, [datasourceName]),
    };
  } else if (containerType === 'module') {
    return {
      query: partial(calcModuleDatasource, [containerName, datasourceName]),
      count: partial(countModuleDatasource, [containerName, datasourceName]),
      cursor: partial(cursorModuleDatasource, [containerName, datasourceName]),
      update: partial(updateModuleDatasource, [containerName, datasourceName]),
      create: partial(createModuleDatasource, [containerName, datasourceName]),
      remove: partial(removeModuleDatasource, [containerName, datasourceName]),
      meta: partial(getModuleDatasourceMeta, [containerName, datasourceName]),
      queryMeta: partial(calcModuleDatasourceMeta, [containerName, datasourceName]),
      countMeta: partial(countModuleDatasourceMeta, [containerName, datasourceName]),
      cursorMeta: partial(cursorModuleDatasourceMeta, [containerName, datasourceName]),
      updateMeta: partial(updateModuleDatasourceMeta, [containerName, datasourceName]),
      createMeta: partial(createModuleDatasourceMeta, [containerName, datasourceName]),
      removeMeta: partial(removeModuleDatasourceMeta, [containerName, datasourceName]),
      validateUpdate: partial(validateUpdateModuleDatasource, [containerName, datasourceName]),
      validateCreate: partial(validateCreateModuleDatasource, [containerName, datasourceName]),
      validateRemove: partial(validateRemoveModuleDatasource, [containerName, datasourceName]),
      feature: partial(getModuleDatasourceFeature, [containerName, datasourceName]),
      invoke: partial(invokeModuleDatasource, [containerName, datasourceName]),
    };
  } else {
    throw new Error(`Unsupported containerType: ${containerType}`);
  }
};

export async function calcGlobalDatasource(name, { pst, psz, filters = [], sortBys = [], sortTypes = [], params = {}, dmPath }) {
  return request(`${config.apiContextPath}/api/datasource/user/${encodeURIComponent(name)}`, makeParams({ pst, psz, dmPath }, { filters, sortBys, sortTypes, params }));
}

export async function calcGlobalDatasourceMeta(name, { params = {}, dmPath }) {
  return request(`${config.apiContextPath}/api/datasource/user/${encodeURIComponent(name)}/meta/query`, makeParams({ dmPath }, { params }));
}

export async function countGlobalDatasource(name, { filters = [], params = {}, dmPath }) {
  return request(`${config.apiContextPath}/api/datasource/user/${encodeURIComponent(name)}/count`, makeParams({ dmPath }, { filters, params }));
}

export async function countGlobalDatasourceMeta(name, { params = {}, dmPath }) {
  return request(`${config.apiContextPath}/api/datasource/user/${encodeURIComponent(name)}/meta/count`, makeParams({ dmPath }, { params }));
}

export async function cursorGlobalDatasource(name, key, params = {}, dmPath) {
  return request(`${config.apiContextPath}/api/datasource/user/${encodeURIComponent(name)}/cursor`, makeParams({ key, dmPath }, { params }));
}

export async function cursorGlobalDatasourceMeta(name, key, params = {}, dmPath) {
  return request(`${config.apiContextPath}/api/datasource/user/${encodeURIComponent(name)}/meta/cursor`, makeParams({ key, dmPath }, { params }));
}

export async function validateUpdateGlobalDatasource(name, key, params = {}, dmPath) {
  return request(`${config.apiContextPath}/api/datasource/user/${encodeURIComponent(name)}/update/validate`, makeParams({ key, dmPath }, { params }));
}

export async function updateGlobalDatasource(name, key, params = {}, dmPath) {
  return post(`${config.apiContextPath}/api/datasource/user/${encodeURIComponent(name)}/update`, {
    key,
    params,
    dmPath,
  });
}

export async function updateGlobalDatasourceMeta(name, key, params = {}, dmPath) {
  return post(`${config.apiContextPath}/api/datasource/user/${encodeURIComponent(name)}/meta/update`, {
    key,
    params,
    dmPath,
  });
}

export async function validateCreateGlobalDatasource(name, params = {}, dmPath) {
  return request(`${config.apiContextPath}/api/datasource/user/${encodeURIComponent(name)}/create/validate`, makeParams({ dmPath }, { params }));
}

export async function createGlobalDatasource(name, params = {}, dmPath) {
  return post(`${config.apiContextPath}/api/datasource/user/${encodeURIComponent(name)}/create`, {
    params,
    dmPath,
  });
}

export async function createGlobalDatasourceMeta(name, params = {}, dmPath) {
  return post(`${config.apiContextPath}/api/datasource/user/${encodeURIComponent(name)}/meta/create`, {
    params,
    dmPath,
  });
}

export async function validateRemoveGlobalDatasource(name, key, params = {}, dmPath) {
  return request(`${config.apiContextPath}/api/datasource/user/${encodeURIComponent(name)}/remove/validate`, makeParams({ key, dmPath }, { params }));
}

export async function removeGlobalDatasource(name, key, params = {}, dmPath) {
  return post(`${config.apiContextPath}/api/datasource/user/${encodeURIComponent(name)}/remove`, {
    key,
    params,
    dmPath,
  });
}

export async function removeGlobalDatasourceMeta(name, key, params = {}, dmPath) {
  return post(`${config.apiContextPath}/api/datasource/user/${encodeURIComponent(name)}/meta/remove`, {
    key,
    params,
    dmPath,
  });
}

export async function getGlobalDatasourceMeta(name, dmPath) {
  return request(`${config.apiContextPath}/api/datasource/user/${encodeURIComponent(name)}/meta`, makeParams({ dmPath }, {}));
}

export async function getGlobalDatasourceFeature(name, dmPath) {
  return request(`${config.apiContextPath}/api/datasource/user/${encodeURIComponent(name)}/feature`, makeParams({ dmPath }, {}));
}

export async function invokeGlobalDatasource(dsName, opName, params = {}, dmPath, { pst, psz, filters = [], includeKeys, excludeKeys, termWhenFail = true }) {
  return post(`${config.apiContextPath}/api/datasource/user/${encodeURIComponent(dsName)}/operation/${encodeURIComponent(opName)}/invoke`, {
    includeKeys,
    excludeKeys,
  }, makeParams({ twf: termWhenFail ? 1 : 0, pst, psz }, { filters, params }));
}

export async function calcModuleDatasource(mdName, dsName, { pst, psz, filters = [], sortBys = [], sortTypes = [], params = {}, dmPath }) {
  return request(`${config.apiContextPath}/api/module/user/${encodeURIComponent(mdName)}/datasource/${encodeURIComponent(dsName)}`, makeParams({ pst, psz, dmPath }, { filters, sortBys, sortTypes, params }));
}

export async function calcModuleDatasourceMeta(mdName, dsName, { params = {}, dmPath }) {
  return request(`${config.apiContextPath}/api/module/user/${encodeURIComponent(mdName)}/datasource/${encodeURIComponent(dsName)}/meta/query`, makeParams({ dmPath }, { params }));
}

export async function countModuleDatasource(mdName, dsName, { filters = [], params = {}, dmPath }) {
  return request(`${config.apiContextPath}/api/module/user/${encodeURIComponent(mdName)}/datasource/${encodeURIComponent(dsName)}/count`, makeParams({ dmPath }, { filters, params }));
}

export async function countModuleDatasourceMeta(mdName, dsName, { params = {}, dmPath }) {
  return request(`${config.apiContextPath}/api/module/user/${encodeURIComponent(mdName)}/datasource/${encodeURIComponent(dsName)}/meta/count`, makeParams({ dmPath }, { params }));
}

export async function cursorModuleDatasource(mdName, dsName, key, params = {}, dmPath) {
  return request(`${config.apiContextPath}/api/module/user/${encodeURIComponent(mdName)}/datasource/${encodeURIComponent(dsName)}/cursor`, makeParams({ key, dmPath }, { params }));
}

export async function cursorModuleDatasourceMeta(mdName, dsName, key, params = {}, dmPath) {
  return request(`${config.apiContextPath}/api/module/user/${encodeURIComponent(mdName)}/datasource/${encodeURIComponent(dsName)}/meta/cursor`, makeParams({ key, dmPath }, { params }));
}

export async function validateUpdateModuleDatasource(mdName, dsName, key, params = {}, dmPath) {
  return request(`${config.apiContextPath}/api/module/user/${encodeURIComponent(mdName)}/datasource/${encodeURIComponent(dsName)}/update/validate`, makeParams({ key, dmPath }, { params }));
}

export async function updateModuleDatasource(mdName, dsName, key, params = {}, dmPath) {
  return post(`${config.apiContextPath}/api/module/user/${encodeURIComponent(mdName)}/datasource/${encodeURIComponent(dsName)}/update`, {
    key,
    params,
    dmPath,
  });
}

export async function updateModuleDatasourceMeta(mdName, dsName, key, params = {}, dmPath) {
  return post(`${config.apiContextPath}/api/module/user/${encodeURIComponent(mdName)}/datasource/${encodeURIComponent(dsName)}/meta/update`, {
    key,
    params,
    dmPath,
  });
}

export async function validateCreateModuleDatasource(mdName, dsName, params = {}, dmPath) {
  return request(`${config.apiContextPath}/api/module/user/${encodeURIComponent(mdName)}/datasource/${encodeURIComponent(dsName)}/create/validate`, makeParams({ dmPath }, { params }));
}

export async function createModuleDatasource(mdName, dsName, params = {}, dmPath) {
  return post(`${config.apiContextPath}/api/module/user/${encodeURIComponent(mdName)}/datasource/${encodeURIComponent(dsName)}/create`, {
    params,
    dmPath,
  });
}

export async function createModuleDatasourceMeta(mdName, dsName, params = {}, dmPath) {
  return post(`${config.apiContextPath}/api/module/user/${encodeURIComponent(mdName)}/datasource/${encodeURIComponent(dsName)}/meta/create`, {
    params,
    dmPath,
  });
}

export async function validateRemoveModuleDatasource(mdName, dsName, key, params = {}, dmPath) {
  return request(`${config.apiContextPath}/api/module/user/${encodeURIComponent(mdName)}/datasource/${encodeURIComponent(dsName)}/remove/validate`, makeParams({ key, dmPath }, { params }));
}

export async function removeModuleDatasource(mdName, dsName, key, params = {}, dmPath) {
  return post(`${config.apiContextPath}/api/module/user/${encodeURIComponent(mdName)}/datasource/${encodeURIComponent(dsName)}/remove`, {
    key,
    params,
    dmPath,
  });
}

export async function removeModuleDatasourceMeta(mdName, dsName, key, params = {}, dmPath) {
  return post(`${config.apiContextPath}/api/module/user/${encodeURIComponent(mdName)}/datasource/${encodeURIComponent(dsName)}/meta/remove`, {
    key,
    params,
    dmPath,
  });
}

export async function getModuleDatasourceMeta(mdName, dsName, dmPath) {
  return request(`${config.apiContextPath}/api/module/user/${encodeURIComponent(mdName)}/datasource/${encodeURIComponent(dsName)}/meta`, makeParams({ dmPath }, {}));
}

export async function getModuleDatasourceFeature(mdName, dsName, dmPath) {
  return request(`${config.apiContextPath}/api/module/user/${encodeURIComponent(mdName)}/datasource/${encodeURIComponent(dsName)}/feature`, makeParams({ dmPath }, {}));
}

export async function invokeModuleDatasource(mdName, dsName, opName, params = {}, dmPath, { pst, psz, filters = [], includeKeys, excludeKeys, termWhenFail = true }) {
  return post(`${config.apiContextPath}/api/module/user/${encodeURIComponent(mdName)}/datasource/${encodeURIComponent(dsName)}/operation/${encodeURIComponent(opName)}/invoke`, {
    includeKeys,
    excludeKeys,
  }, makeParams({ twf: termWhenFail ? 1 : 0, pst, psz }, { filters, params }));
}

export class Datasource {
  constructor(coordinate, dmPath) {
    this.api = datasourceApi(coordinate);
    this.dmPath = dmPath;
    this.dealWithPath = ::this.dealWithPath;
    this.query = ::this.query;
    this.queryMeta = ::this.queryMeta;
    this.count = ::this.count;
    this.countMeta = ::this.countMeta;
    this.cursor = ::this.cursor;
    this.cursorMeta = ::this.cursorMeta;
    this.update = ::this.update;
    this.updateMeta = ::this.updateMeta;
    this.validateUpdate = ::this.validateUpdate;
    this.create = ::this.create;
    this.createMeta = ::this.createMeta;
    this.validateCreate = ::this.validateCreate;
    this.remove = ::this.remove;
    this.removeMeta = ::this.removeMeta;
    this.validateRemove = ::this.validateRemove;
  }
  async dealWithPath() {
    if (this.dmPath) {
      return this.dmPath;
    } else {
      const domain = await getDomain();
      return domain.path;
    }
  }
  async query(params = {}, pst = 0, psz = -1, { filters = [], sortBys = [], sortTypes = [] }) {
    const dmPath = await this.dealWithPath();
    return this.api.query({ pst, psz, filters, sortBys, sortTypes, params, dmPath });
  }
  async queryMeta(params = {}) {
    const dmPath = await this.dealWithPath();
    return this.api.query({ params, dmPath });
  }
  async count(params = {}, { filters = [] }) {
    const dmPath = await this.dealWithPath();
    return this.api.count({ filters, params, dmPath });
  }
  async countMeta(params = {}) {
    const dmPath = await this.dealWithPath();
    return this.api.countMeta({ params, dmPath });
  }
  async cursor(key, params = {}) {
    const dmPath = await this.dealWithPath();
    return this.api.cursor(key, params, dmPath);
  }
  async cursorMeta(key, params = {}) {
    const dmPath = await this.dealWithPath();
    return this.api.cursorMeta(key, params, dmPath);
  }
  async update(key, params = {}) {
    const dmPath = await this.dealWithPath();
    return this.api.update(key, params, dmPath);
  }
  async updateMeta(key, params = {}) {
    const dmPath = await this.dealWithPath();
    return this.api.updateMeta(key, params, dmPath);
  }
  async validateUpdate(key, params = {}) {
    const dmPath = await this.dealWithPath();
    return this.api.validateUpdate(key, params, dmPath);
  }
  async create(params = {}) {
    const dmPath = await this.dealWithPath();
    return this.api.create(params, dmPath);
  }
  async createMeta(params = {}) {
    const dmPath = await this.dealWithPath();
    return this.api.createMeta(params, dmPath);
  }
  async validateCreate(params = {}) {
    const dmPath = await this.dealWithPath();
    return this.api.validateCreate(params, dmPath);
  }
  async remove(key, params = {}) {
    const dmPath = await this.dealWithPath();
    return this.api.remove(key, params, dmPath);
  }
  async removeMeta(key, params = {}) {
    const dmPath = await this.dealWithPath();
    return this.api.removeMeta(key, params, dmPath);
  }
  async validateRemove(key, params = {}) {
    const dmPath = await this.dealWithPath();
    return this.api.validateRemove(key, params, dmPath);
  }
  async feature() {
    const dmPath = await this.dealWithPath();
    return this.api.feature(dmPath);
  }
  async invoke(name, params = {}, { pst, psz, filters = [], includeKeys, excludeKeys, termWhenFail = true }) {
    const dmPath = await this.dealWithPath();
    return this.api.invoke(name, params, dmPath, { pst, psz, filters, includeKeys, excludeKeys, termWhenFail });
  }
}
